package com.ym.mec.biz.dal.wrapper;

import com.alibaba.fastjson.JSON;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

import java.math.BigDecimal;
import java.util.List;
import java.util.Optional;

import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.commons.lang3.StringUtils;

/**
 * 机构学练宝配置表
 * 2024-10-23 10:48:21
 */
@ApiModel(value = "TenantMemberRankSettingWrapper对象", description = "机构学练宝配置表查询对象")
public class TenantMemberRankSettingWrapper {

    @Data
	@Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel(" TenantMemberRankSettingQuery-机构学练宝配置表")
    public static class TenantMemberRankSettingQuery implements QueryInfo {
    
    	@ApiModelProperty("当前页")
        private Integer page;
        
        @ApiModelProperty("分页行数")
        private Integer rows;
        
        @ApiModelProperty("关键字匹配")
		private String keyword;
        
        public String getKeyword() {
            return Optional.ofNullable(keyword).filter(StringUtils::isNotBlank).orElse(null);
        }
        
        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static TenantMemberRankSettingQuery from(String json) {
            return JSON.parseObject(json, TenantMemberRankSettingQuery.class);
        }
    }  

    @Data
	@ApiModel(" TenantMemberRankSettingAdd-机构学练宝配置表")
    public static class TenantMemberRankSettingAdd {

        @ApiModelProperty("学练宝ID")
        private Integer memberRankSettingId;

        @ApiModelProperty("学练宝名称")
        private String memberRankSettingName;

        @ApiModelProperty("机构配置：TENANT, 学生配置：STUDENT")
        private String type;

        @ApiModelProperty("学练宝配置")
        private MemberConfig memberConfig;
        
        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static TenantMemberRankSettingAdd from(String json) {
            return JSON.parseObject(json, TenantMemberRankSettingAdd.class);
        }
	}


    @Data
    @ApiModel(" MemberConfig")
    public static class MemberConfig {

//        "day_divide": "0",
//            "month_divide": "0",
//            "quarter_divide": "1",
//            "half_year_divide": "0",
//            "year_divide": "0"

        @ApiModelProperty("日分成")
        private BigDecimal dayDivide;

        @ApiModelProperty("月分成")
        private BigDecimal monthDivide;

        @ApiModelProperty("季度分成")
        private BigDecimal quarterDivide;

        @ApiModelProperty("半年分成")
        private BigDecimal halfYearDivide;

        @ApiModelProperty("年分成")
        private BigDecimal yearDivide;

    }
}