package com.ym.mec.biz.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.dayaedu.cbs.common.enums.music.EMusicSourceType;
import com.dayaedu.cbs.common.enums.school.EMusicSheetType;
import com.dayaedu.cbs.openfeign.service.CbsMusicSheetService;
import com.dayaedu.cbs.openfeign.wrapper.music.*;
import com.google.common.collect.Lists;
import com.microsvc.toolkit.common.response.paging.PageInfo;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.ym.mec.biz.dal.dao.SysUserTenantDao;
import com.ym.mec.biz.dal.entity.Instrument;
import com.ym.mec.biz.dal.entity.SysMusicScore;
import com.ym.mec.biz.dal.enums.ClientEnum;
import com.ym.mec.biz.dal.wrapper.MusicSheetWrapper;
import com.ym.mec.biz.service.InstrumentService;
import com.ym.mec.biz.service.SysMusicScoreCategoriesService;
import com.ym.mec.biz.service.SysMusicScoreService;
import com.ym.mec.common.exception.BizException;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class CbsMusicSheetServiceImpl implements CbsMusicSheetService {

    private static final Logger log = LoggerFactory.getLogger(CbsMusicSheetServiceImpl.class);

    @Resource
    private SysMusicScoreService sysMusicScoreService;
    @Resource
    private SysMusicScoreCategoriesService sysMusicScoreCategoriesService;
    @Resource
    private SysUserTenantDao sysUserTenantDao;
    @Resource
    private InstrumentService instrumentService;

    @Autowired
    private SysMusicScoreService musicSheetService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean saveApplicationExtend(List<CbsMusicSheetApplicationExtendWrapper.MusicSheetApplicationExtend> addMusicSheet) {
        if (CollectionUtils.isEmpty(addMusicSheet)) {
            throw new BizException("请选择曲目信息");
        }
        List<Long> cbsMusicSheetIds = addMusicSheet.stream().map(CbsMusicSheetApplicationExtendWrapper.MusicSheetApplicationExtend::getMusicSheetId).collect(Collectors.toList());
        for (CbsMusicSheetApplicationExtendWrapper.MusicSheetApplicationExtend extend : addMusicSheet) {
            CbsMusicSheetApiWrapper.MusicSheetUpdate musicSheetUpdate = extend.getMusicSheetUpdate();
            if(StringUtils.isNotBlank(musicSheetUpdate.getMusicalInstrumentIds())) {
                List<Instrument> instruments = instrumentService.lambdaQuery()
                        .in(Instrument::getId, Arrays.stream(musicSheetUpdate.getMusicalInstrumentIds().split(","))
                                .map(Integer::parseInt).collect(Collectors.toList())).list();
                if (CollectionUtils.isNotEmpty(instruments)) {
                    musicSheetUpdate.setSubjectIds(instruments.stream().map(e -> e.getSubjectId().toString()).distinct().collect(Collectors.joining(",")));
                }
            }
        }
        //需要修改的曲目
        List<SysMusicScore> musicSheets = sysMusicScoreService.getDao().findByCbsId(cbsMusicSheetIds,null);
        if (CollectionUtils.isNotEmpty(musicSheets)){
            List<String> collect = musicSheets.stream().map(SysMusicScore::getCbsMusicSheetId).collect(Collectors.toList());
            //获取需要修改的曲目
            List<CbsMusicSheetApplicationExtendWrapper.MusicSheetApplicationExtend> updateMusicScore =
                    addMusicSheet.stream().filter(e -> collect.contains(e.getMusicSheetId().toString())).collect(Collectors.toList());
            sysMusicScoreService.getDao().updateBatchByCbsId(updateMusicScore);
            //排除已经存在的曲目
            addMusicSheet = addMusicSheet.stream().filter(e -> !collect.contains(e.getMusicSheetId().toString())).collect(Collectors.toList());
        }
        if(CollectionUtils.isNotEmpty(addMusicSheet)){
            sysMusicScoreService.getDao().batchInsert(addMusicSheet);
            List<Long> cbsIds = addMusicSheet.stream().map(CbsMusicSheetApplicationExtendWrapper.MusicSheetApplicationExtend::getMusicSheetId).collect(Collectors.toList());
            List<SysMusicScore> sysMusicScores = sysMusicScoreService.getDao().findByCbsId(cbsIds,false);
            List<Integer> collect = sysMusicScores.stream().map(SysMusicScore::getId).collect(Collectors.toList());
            sysMusicScoreService.getDao().batchInsertAccompaniment(collect);
        }
        return true;
    }

    public Boolean updateApplicationExtendStatus(CbsMusicSheetApplicationExtendWrapper.MusicSheetApplicationExtendStatus updateMusicSheet){
        if (updateMusicSheet == null || StringUtils.isEmpty(updateMusicSheet.getIds())){
            throw new BizException("请选择曲目");
        }
        if (updateMusicSheet.getStatus() == null){
            throw new BizException("请选择状态");
        }
        sysMusicScoreService.getDao().batchEnable(updateMusicSheet.getIds(),updateMusicSheet.getStatus());
        return true;
    }

    public Boolean delApplicationExtend(CbsMusicSheetApplicationExtendWrapper.MusicSheetApplicationExtendDel musicSheetApplicationExtendDel){
        if (musicSheetApplicationExtendDel == null || StringUtils.isEmpty(musicSheetApplicationExtendDel.getIds())){
            throw new BizException("请选择曲目");
        }
        sysMusicScoreService.getDao().batchDel(musicSheetApplicationExtendDel.getIds());
        return true;
    }

    public List<CbsMusicSheetTagWrapper.MusicSheetTag> tagList(CbsMusicSheetTagWrapper.MusicSheetTagQuery queryInfo){
        return null;
    }

    public List<CbsMusicSheetCategoryWrapper.MusicSheetCategory> categoryList(CbsMusicSheetCategoryWrapper.MusicSheetCategoryQuery queryInfo){
        return sysMusicScoreCategoriesService.getDao().categoryList(queryInfo);
    }

    public PageInfo<CbsMusicSheetExtendWrapper.MusicSheetExtend> ownerList(CbsMusicSheetExtendWrapper.MusicSheetExtendQuery queryInfo){
        IPage<CbsMusicSheetExtendWrapper.MusicSheetExtend> page = QueryInfo.getPage(queryInfo);
        if (queryInfo.getSourceType() == null){
            queryInfo.setSourceType(EMusicSourceType.PERSON);
        }
        if(queryInfo.getSourceType() == EMusicSourceType.PERSON){
            return QueryInfo.pageInfo(sysUserTenantDao.personOwnerList(page, queryInfo));
        }else {
            return QueryInfo.pageInfo(sysUserTenantDao.orgOwnerList(page, queryInfo));
        }
    }

    public boolean updateMusicSheet(CbsMusicSheetApiWrapper.MusicSheetUpdate musicSheetUpdate) {
        if(StringUtils.isNotBlank(musicSheetUpdate.getMusicalInstrumentIds())) {
            List<Instrument> instruments = instrumentService.lambdaQuery()
                    .in(Instrument::getId, Arrays.stream(musicSheetUpdate.getMusicalInstrumentIds().split(","))
                            .map(Integer::parseInt).collect(Collectors.toList())).list();
            if (CollectionUtils.isNotEmpty(instruments)) {
                musicSheetUpdate.setSubjectIds(instruments.stream().map(e -> e.getSubjectId().toString()).distinct().collect(Collectors.joining(",")));
            }
        }
        sysMusicScoreService.getDao().updateMusicSheet(musicSheetUpdate);
        return true;
    }

    /**
     * 获取曲目应用拓展详情
     * @param query CbsMusicSheetAppWrapper.MusicSheetQuery
     * @return CbsMusicSheetAppWrapper.MusicSheetDetail
     */
    @Override
    public CbsMusicSheetAppWrapper.MusicSheetDetail getMusicSheetDetail(CbsMusicSheetAppWrapper.MusicSheetQuery query) {

        // 根据总控曲目ID，查询应用端曲目
        List<SysMusicScore> musicScores = sysMusicScoreService.getDao().findByCbsId(Lists.newArrayList(query.getCbsMusicId()), false);
        if (org.apache.commons.collections.CollectionUtils.isEmpty(musicScores)) {
            throw new BizException("曲目不存在");
        }

        // 返回应用端曲目
        SysMusicScore musicScore = musicScores.get(0);
        CbsMusicSheetAppWrapper.MusicSheetDetail music = CbsMusicSheetAppWrapper.MusicSheetDetail
            .builder()
            .bizId(musicScore.getId().longValue())
            .paymentType(StringUtils.isBlank(musicScore.getRankIds()) ? "FREE" : "VIP")
            .bizMusicCategoryId(musicScore.getCategoriesId() == null ? null : musicScore.getCategoriesId().longValue())
            .scoreType(musicScore.getScoreType())
            .isConvertibleScore(musicScore.getIsConvertibleScore())
            .specialPercussionFlag(false)
            .build();

        // 如果是合奏 并且乐器ID = 2268
        if ("2268".equals(query.getMusicalInstrumentIds())
            && musicScore.getMusicSheetType().equals(EMusicSheetType.CONCERT.getCode())) {
            // 设置特殊打击乐标识
            music.setSpecialPercussionFlag(true);
        }

        // 学生端查询曲目使用状态
        if (query.getClientType().equals(ClientEnum.STUDENT.getCode())) {
            MusicSheetWrapper.ClientMusicSheetInfoQuery clientQuery = MusicSheetWrapper.ClientMusicSheetInfoQuery
                .builder()
                .userType(ClientEnum.STUDENT)
                .userId(query.getUserId().intValue())
                .musicSheetId(musicScore.getId())
                .build();
            MusicSheetWrapper.ClientMusicSheetInfo clientMusicSheetInfo = musicSheetService.queryMusicScoreDetail(clientQuery);
            if (clientMusicSheetInfo != null) {
                music.setUseStatus(clientMusicSheetInfo.getUseStatus());
            } else {
                music.setUseStatus("LOCK");
            }
        }

        return music;
    }
}